@@ -665,6 +665,10 @@ void QQuickRangeSlider::setFrom(qreal from)
     if (isComponentComplete()) {
         d->first->setValue(d->first->value());
         d->second->setValue(d->second->value());
+        auto *firstPrivate = QQuickRangeSliderNodePrivate::get(d->first);
+        auto *secondPrivate = QQuickRangeSliderNodePrivate::get(d->second);
+        firstPrivate->updatePosition(true);
+        secondPrivate->updatePosition();
     }
 }
 
@@ -693,6 +697,10 @@ void QQuickRangeSlider::setTo(qreal to)
     if (isComponentComplete()) {
         d->first->setValue(d->first->value());
         d->second->setValue(d->second->value());
+        auto *firstPrivate = QQuickRangeSliderNodePrivate::get(d->first);
+        auto *secondPrivate = QQuickRangeSliderNodePrivate::get(d->second);
+        firstPrivate->updatePosition(true);
+        secondPrivate->updatePosition();
     }
 }
 
@@ -746,7 +754,6 @@ qreal QQuickRangeSlider::valueAt(qreal position) const
 }
 
 /*!
-    \qmlpropertygroup QtQuick.Controls::RangeSlider::first
     \qmlproperty real QtQuick.Controls::RangeSlider::first.value
     \qmlproperty real QtQuick.Controls::RangeSlider::first.position
     \qmlproperty real QtQuick.Controls::RangeSlider::first.visualPosition
@@ -825,7 +832,6 @@ QQuickRangeSliderNode *QQuickRangeSlider::first() const
 */
 
 /*!
-    \qmlpropertygroup QtQuick.Controls::RangeSlider::second
     \qmlproperty real QtQuick.Controls::RangeSlider::second.value
     \qmlproperty real QtQuick.Controls::RangeSlider::second.position
     \qmlproperty real QtQuick.Controls::RangeSlider::second.visualPosition
