@@ -55,7 +55,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype TextArea
     \inherits TextEdit
-    \instantiates QQuickTextArea
+//!     \instantiates QQuickTextArea
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-input
@@ -96,6 +96,24 @@ QT_BEGIN_NAMESPACE
     \li Clips the content
     \endlist
 
+    \section2 Tab Focus
+
+    By default, pressing the tab key while TextArea has
+    \l {Item::activeFocus}{active focus} results in a tab character being input
+    into the control itself. To make tab pass active focus onto another item,
+    use the attached \l KeyNavigation properties:
+
+    \code
+    TextField {
+        id: textField
+    }
+
+    TextArea {
+        KeyNavigation.priority: KeyNavigation.BeforeItem
+        KeyNavigation.tab: textField
+    }
+    \endcode
+
     \sa TextField, {Customizing TextArea}, {Input Controls}
 */
 
@@ -103,8 +121,8 @@ QT_BEGIN_NAMESPACE
     \qmlsignal QtQuick.Controls::TextArea::pressAndHold(MouseEvent event)
 
     This signal is emitted when there is a long press (the delay depends on the platform plugin).
-    The \l {MouseEvent}{event} parameter provides information about the press, including the x and y
-    position of the press, and which button is pressed.
+    The \a event parameter provides information about the press, including the x and y
+    coordinates of the press, and which button is pressed.
 
     \sa pressed, released
 */
@@ -114,8 +132,8 @@ QT_BEGIN_NAMESPACE
     \since QtQuick.Controls 2.1 (Qt 5.8)
 
     This signal is emitted when the text area is pressed by the user.
-    The \l {MouseEvent}{event} parameter provides information about the press,
-    including the x and y position of the press, and which button is pressed.
+    The \a event parameter provides information about the press,
+    including the x and y coordinates of the press, and which button is pressed.
 
     \sa released, pressAndHold
 */
@@ -125,22 +143,14 @@ QT_BEGIN_NAMESPACE
     \since QtQuick.Controls 2.1 (Qt 5.8)
 
     This signal is emitted when the text area is released by the user.
-    The \l {MouseEvent}{event} parameter provides information about the release,
-    including the x and y position of the press, and which button is pressed.
+    The \a event parameter provides information about the release,
+    including the x and y coordinates of the press, and which button
+    is pressed.
 
     \sa pressed, pressAndHold
 */
 
 QQuickTextAreaPrivate::QQuickTextAreaPrivate()
-    : QQuickTextEditPrivate(),
-#if QT_CONFIG(quicktemplates2_hover)
-      hovered(false),
-      explicitHoverEnabled(false),
-#endif
-      background(nullptr),
-      focusReason(Qt::OtherFocusReason),
-      accessibleAttached(nullptr),
-      flickable(nullptr)
 {
 #if QT_CONFIG(accessibility)
     QAccessible::installActivationObserver(this);
@@ -154,28 +164,85 @@ QQuickTextAreaPrivate::~QQuickTextAreaPrivate()
 #endif
 }
 
-void QQuickTextAreaPrivate::resizeBackground()
+void QQuickTextAreaPrivate::setTopInset(qreal value, bool reset)
 {
     Q_Q(QQuickTextArea);
-    if (background) {
-        QQuickItemPrivate *p = QQuickItemPrivate::get(background);
-        if (!p->widthValid && qFuzzyIsNull(background->x())) {
-            if (flickable)
-                background->setWidth(flickable->width());
-            else
-                background->setWidth(q->width());
-            p->widthValid = false;
-        }
-        if (!p->heightValid && qFuzzyIsNull(background->y())) {
-            if (flickable)
-                background->setHeight(flickable->height());
-            else
-                background->setHeight(q->height());
-            p->heightValid = false;
-        }
+    const QMarginsF oldInset = getInset();
+    extra.value().topInset = value;
+    extra.value().hasTopInset = !reset;
+    if (!qFuzzyCompare(oldInset.top(), value)) {
+        emit q->topInsetChanged();
+        q->insetChange(getInset(), oldInset);
     }
 }
 
+void QQuickTextAreaPrivate::setLeftInset(qreal value, bool reset)
+{
+    Q_Q(QQuickTextArea);
+    const QMarginsF oldInset = getInset();
+    extra.value().leftInset = value;
+    extra.value().hasLeftInset = !reset;
+    if (!qFuzzyCompare(oldInset.left(), value)) {
+        emit q->leftInsetChanged();
+        q->insetChange(getInset(), oldInset);
+    }
+}
+
+void QQuickTextAreaPrivate::setRightInset(qreal value, bool reset)
+{
+    Q_Q(QQuickTextArea);
+    const QMarginsF oldInset = getInset();
+    extra.value().rightInset = value;
+    extra.value().hasRightInset = !reset;
+    if (!qFuzzyCompare(oldInset.right(), value)) {
+        emit q->rightInsetChanged();
+        q->insetChange(getInset(), oldInset);
+    }
+}
+
+void QQuickTextAreaPrivate::setBottomInset(qreal value, bool reset)
+{
+    Q_Q(QQuickTextArea);
+    const QMarginsF oldInset = getInset();
+    extra.value().bottomInset = value;
+    extra.value().hasBottomInset = !reset;
+    if (!qFuzzyCompare(oldInset.bottom(), value)) {
+        emit q->bottomInsetChanged();
+        q->insetChange(getInset(), oldInset);
+    }
+}
+
+void QQuickTextAreaPrivate::resizeBackground()
+{
+    if (!background)
+        return;
+
+    resizingBackground = true;
+
+    // When using the attached property TextArea.flickable, we reparent the background out
+    // of TextArea and into the Flickable since we don't want the background to move while
+    // flicking. This means that the size of the background should also follow the size of
+    // the Flickable rather than the size of the TextArea.
+    const auto flickable = qobject_cast<QQuickFlickable *>(background->parentItem());
+
+    QQuickItemPrivate *p = QQuickItemPrivate::get(background);
+    if (((!p->widthValid || !extra.isAllocated() || !extra->hasBackgroundWidth) && qFuzzyIsNull(background->x()))
+            || (extra.isAllocated() && (extra->hasLeftInset || extra->hasRightInset))) {
+        const qreal bgWidth = flickable ? flickable->width() : width;
+        background->setX(getLeftInset());
+        background->setWidth(bgWidth - getLeftInset() - getRightInset());
+    }
+
+    if (((!p->heightValid || !extra.isAllocated() || !extra->hasBackgroundHeight) && qFuzzyIsNull(background->y()))
+            || (extra.isAllocated() && (extra->hasTopInset || extra->hasBottomInset))) {
+        const qreal bgHeight = flickable ? flickable->height() : height;
+        background->setY(getTopInset());
+        background->setHeight(bgHeight - getTopInset() - getBottomInset());
+    }
+
+    resizingBackground = false;
+}
+
 /*!
     \internal
 
@@ -190,21 +257,74 @@ void QQuickTextAreaPrivate::resolveFont()
     inheritFont(QQuickControlPrivate::parentFont(q));
 }
 
-void QQuickTextAreaPrivate::inheritFont(const QFont &f)
+void QQuickTextAreaPrivate::inheritFont(const QFont &font)
+{
+    QFont parentFont = extra.isAllocated() ? extra->requestedFont.resolve(font) : font;
+    parentFont.resolve(extra.isAllocated() ? extra->requestedFont.resolve() | font.resolve() : font.resolve());
+
+    const QFont defaultFont = QQuickTheme::font(QQuickTheme::TextArea);
+    QFont resolvedFont = parentFont.resolve(defaultFont);
+    // See comment in QQuickControlPrivate::inheritFont
+    if (defaultFont.families().isEmpty())
+        resolvedFont.setFamilies(QStringList());
+
+    setFont_helper(resolvedFont);
+}
+
+/*!
+    \internal
+
+    Assign \a font to this control, and propagate it to all children.
+*/
+void QQuickTextAreaPrivate::updateFont(const QFont &font)
 {
     Q_Q(QQuickTextArea);
-    QFont parentFont = font.resolve(f);
-    parentFont.resolve(font.resolve() | f.resolve());
+    QFont oldFont = sourceFont;
+    q->QQuickTextEdit::setFont(font);
 
-    const QFont defaultFont = QQuickControlPrivate::themeFont(QPlatformTheme::EditorFont);
-    const QFont resolvedFont = parentFont.resolve(defaultFont);
+    QQuickControlPrivate::updateFontRecur(q, font);
 
-    const bool changed = resolvedFont != sourceFont;
-    q->QQuickTextEdit::setFont(resolvedFont);
-    if (changed)
+    if (oldFont != font)
         emit q->fontChanged();
 }
 
+/*!
+    \internal
+
+    Determine which palette is implicitly imposed on this control by its ancestors
+    and QGuiApplication::palette, resolve this against its own palette (attributes from
+    the implicit palette are copied over). Then propagate this palette to this
+    control's children.
+*/
+void QQuickTextAreaPrivate::resolvePalette()
+{
+    Q_Q(QQuickTextArea);
+    inheritPalette(QQuickControlPrivate::parentPalette(q));
+}
+
+void QQuickTextAreaPrivate::inheritPalette(const QPalette &palette)
+{
+    QPalette parentPalette = extra.isAllocated() ? extra->requestedPalette.resolve(palette) : palette;
+    parentPalette.resolve(extra.isAllocated() ? extra->requestedPalette.resolve() | palette.resolve() : palette.resolve());
+
+    const QPalette defaultPalette = QQuickTheme::palette(QQuickTheme::TextArea);
+    const QPalette resolvedPalette = parentPalette.resolve(defaultPalette);
+
+    setPalette_helper(resolvedPalette);
+}
+
+void QQuickTextAreaPrivate::updatePalette(const QPalette &palette)
+{
+    Q_Q(QQuickTextArea);
+    QPalette oldPalette = resolvedPalette;
+    resolvedPalette = palette;
+
+    QQuickControlPrivate::updatePaletteRecur(q, palette);
+
+    if (oldPalette != palette)
+        emit q->paletteChanged();
+}
+
 #if QT_CONFIG(quicktemplates2_hover)
 void QQuickTextAreaPrivate::updateHoverEnabled(bool enabled, bool xplicit)
 {
@@ -238,6 +358,7 @@ void QQuickTextAreaPrivate::attachFlickable(QQuickFlickable *item)
     QObject::connect(flickable, &QQuickFlickable::contentYChanged, q, &QQuickItem::update);
 
     QQuickItemPrivate::get(flickable)->updateOrAddGeometryChangeListener(this, QQuickGeometryChange::Size);
+    QQuickItemPrivate::get(flickable)->addItemChangeListener(this, QQuickItemPrivate::Destroyed);
     QObjectPrivate::connect(flickable, &QQuickFlickable::contentWidthChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
     QObjectPrivate::connect(flickable, &QQuickFlickable::contentHeightChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
 
@@ -258,10 +379,13 @@ void QQuickTextAreaPrivate::detachFlickable()
     QObject::disconnect(flickable, &QQuickFlickable::contentYChanged, q, &QQuickItem::update);
 
     QQuickItemPrivate::get(flickable)->updateOrRemoveGeometryChangeListener(this, QQuickGeometryChange::Nothing);
+    QQuickItemPrivate::get(flickable)->removeItemChangeListener(this, QQuickItemPrivate::Destroyed);
     QObjectPrivate::disconnect(flickable, &QQuickFlickable::contentWidthChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
     QObjectPrivate::disconnect(flickable, &QQuickFlickable::contentHeightChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
 
     flickable = nullptr;
+
+    resizeBackground();
 }
 
 void QQuickTextAreaPrivate::ensureCursorVisible()
@@ -326,11 +450,21 @@ void QQuickTextAreaPrivate::resizeFlickableContent()
 
 void QQuickTextAreaPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &diff)
 {
-    Q_UNUSED(item);
-    Q_UNUSED(change);
     Q_UNUSED(diff);
+    if (!resizingBackground && item == background) {
+        QQuickItemPrivate *p = QQuickItemPrivate::get(item);
+        // Only set hasBackgroundWidth/Height if it was a width/height change,
+        // otherwise we're prevented from setting a width/height in the future.
+        if (change.widthChange())
+            extra.value().hasBackgroundWidth = p->widthValid;
+        if (change.heightChange())
+            extra.value().hasBackgroundHeight = p->heightValid;
+    }
 
-    resizeFlickableControl();
+    if (flickable)
+        resizeFlickableControl();
+    else
+        resizeBackground();
 }
 
 qreal QQuickTextAreaPrivate::getImplicitWidth() const
@@ -361,7 +495,7 @@ void QQuickTextAreaPrivate::readOnlyChanged(bool isReadOnly)
 {
     Q_UNUSED(isReadOnly);
 #if QT_CONFIG(accessibility)
-    if (accessibleAttached)
+    if (QQuickAccessibleAttached *accessibleAttached = QQuickControlPrivate::accessibleAttached(q_func()))
         accessibleAttached->set_readOnly(isReadOnly);
 #endif
 #if QT_CONFIG(cursor)
@@ -372,18 +506,15 @@ void QQuickTextAreaPrivate::readOnlyChanged(bool isReadOnly)
 #if QT_CONFIG(accessibility)
 void QQuickTextAreaPrivate::accessibilityActiveChanged(bool active)
 {
-    if (accessibleAttached || !active)
+    if (!active)
         return;
 
     Q_Q(QQuickTextArea);
-    accessibleAttached = qobject_cast<QQuickAccessibleAttached *>(qmlAttachedPropertiesObject<QQuickAccessibleAttached>(q, true));
-    if (accessibleAttached) {
-        accessibleAttached->setRole(accessibleRole());
-        accessibleAttached->set_readOnly(q->isReadOnly());
-        accessibleAttached->setDescription(placeholder);
-    } else {
-        qWarning() << "QQuickTextArea: " << q << " QQuickAccessibleAttached object creation failed!";
-    }
+    QQuickAccessibleAttached *accessibleAttached = qobject_cast<QQuickAccessibleAttached *>(qmlAttachedPropertiesObject<QQuickAccessibleAttached>(q, true));
+    Q_ASSERT(accessibleAttached);
+    accessibleAttached->setRole(accessibleRole());
+    accessibleAttached->set_readOnly(q->isReadOnly());
+    accessibleAttached->setDescription(placeholder);
 }
 
 QAccessible::Role QQuickTextAreaPrivate::accessibleRole() const
@@ -412,6 +543,32 @@ void QQuickTextAreaPrivate::executeBackground(bool complete)
         quickCompleteDeferred(q, backgroundName(), background);
 }
 
+void QQuickTextAreaPrivate::itemImplicitWidthChanged(QQuickItem *item)
+{
+    Q_Q(QQuickTextArea);
+    if (item == background)
+        emit q->implicitBackgroundWidthChanged();
+}
+
+void QQuickTextAreaPrivate::itemImplicitHeightChanged(QQuickItem *item)
+{
+    Q_Q(QQuickTextArea);
+    if (item == background)
+        emit q->implicitBackgroundHeightChanged();
+}
+
+void QQuickTextAreaPrivate::itemDestroyed(QQuickItem *item)
+{
+    Q_Q(QQuickTextArea);
+    if (item == background) {
+        background = nullptr;
+        emit q->implicitBackgroundWidthChanged();
+        emit q->implicitBackgroundHeightChanged();
+    } else if (item == flickable) {
+        detachFlickable();
+    }
+}
+
 QQuickTextArea::QQuickTextArea(QQuickItem *parent)
     : QQuickTextEdit(*(new QQuickTextAreaPrivate), parent)
 {
@@ -432,6 +589,7 @@ QQuickTextArea::~QQuickTextArea()
     Q_D(QQuickTextArea);
     if (d->flickable)
         d->detachFlickable();
+    QQuickControlPrivate::removeImplicitSizeListener(d->background, d, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
 }
 
 QQuickTextAreaAttached *QQuickTextArea::qmlAttachedProperties(QObject *object)
@@ -447,10 +605,10 @@ QFont QQuickTextArea::font() const
 void QQuickTextArea::setFont(const QFont &font)
 {
     Q_D(QQuickTextArea);
-    if (d->font.resolve() == font.resolve() && d->font == font)
+    if (d->extra.value().requestedFont.resolve() == font.resolve() && d->extra.value().requestedFont == font)
         return;
 
-    d->font = font;
+    d->extra.value().requestedFont = font;
     d->resolveFont();
 }
 
@@ -480,15 +638,39 @@ void QQuickTextArea::setBackground(QQuickItem *background)
     if (!d->background.isExecuting())
         d->cancelBackground();
 
-    delete d->background;
+    const qreal oldImplicitBackgroundWidth = implicitBackgroundWidth();
+    const qreal oldImplicitBackgroundHeight = implicitBackgroundHeight();
+
+    if (d->extra.isAllocated()) {
+        d->extra.value().hasBackgroundWidth = false;
+        d->extra.value().hasBackgroundHeight = false;
+    }
+
+    QQuickControlPrivate::removeImplicitSizeListener(d->background, d, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
+    QQuickControlPrivate::hideOldItem(d->background);
     d->background = background;
+
     if (background) {
-        background->setParentItem(this);
+        QQuickItemPrivate *p = QQuickItemPrivate::get(background);
+        if (p->widthValid || p->heightValid) {
+            d->extra.value().hasBackgroundWidth = p->widthValid;
+            d->extra.value().hasBackgroundHeight = p->heightValid;
+        }
+        if (d->flickable)
+            background->setParentItem(d->flickable);
+        else
+            background->setParentItem(this);
         if (qFuzzyIsNull(background->z()))
             background->setZ(-1);
         if (isComponentComplete())
             d->resizeBackground();
+        QQuickControlPrivate::addImplicitSizeListener(background, d, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
     }
+
+    if (!qFuzzyCompare(oldImplicitBackgroundWidth, implicitBackgroundWidth()))
+        emit implicitBackgroundWidthChanged();
+    if (!qFuzzyCompare(oldImplicitBackgroundHeight, implicitBackgroundHeight()))
+        emit implicitBackgroundHeightChanged();
     if (!d->background.isExecuting())
         emit backgroundChanged();
 }
@@ -513,12 +695,36 @@ void QQuickTextArea::setPlaceholderText(const QString &text)
 
     d->placeholder = text;
 #if QT_CONFIG(accessibility)
-    if (d->accessibleAttached)
-        d->accessibleAttached->setDescription(text);
+    if (QQuickAccessibleAttached *accessibleAttached = QQuickControlPrivate::accessibleAttached(this))
+        accessibleAttached->setDescription(text);
 #endif
     emit placeholderTextChanged();
 }
 
+/*!
+    \qmlproperty color QtQuick.Controls::TextArea::placeholderTextColor
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+
+    This property holds the color of placeholderText.
+
+    \sa placeholderText
+*/
+QColor QQuickTextArea::placeholderTextColor() const
+{
+    Q_D(const QQuickTextArea);
+    return d->placeholderColor;
+}
+
+void QQuickTextArea::setPlaceholderTextColor(const QColor &color)
+{
+    Q_D(QQuickTextArea);
+    if (d->placeholderColor == color)
+        return;
+
+    d->placeholderColor = color;
+    emit placeholderTextColorChanged();
+}
+
 /*!
     \qmlproperty enumeration QtQuick.Controls::TextArea::focusReason
 
@@ -624,11 +830,186 @@ void QQuickTextArea::resetHoverEnabled()
 #endif
 }
 
+/*!
+    \since QtQuick.Controls 2.3 (Qt 5.10)
+    \qmlproperty palette QtQuick.Controls::TextArea::palette
+
+    This property holds the palette currently set for the text area.
+
+    \sa Control::palette
+*/
+QPalette QQuickTextArea::palette() const
+{
+    Q_D(const QQuickTextArea);
+    QPalette palette = d->resolvedPalette;
+    if (!isEnabled())
+        palette.setCurrentColorGroup(QPalette::Disabled);
+    return palette;
+}
+
+void QQuickTextArea::setPalette(const QPalette &palette)
+{
+    Q_D(QQuickTextArea);
+    if (d->extra.value().requestedPalette.resolve() == palette.resolve() && d->extra.value().requestedPalette == palette)
+        return;
+
+    d->extra.value().requestedPalette = palette;
+    d->resolvePalette();
+}
+
+void QQuickTextArea::resetPalette()
+{
+    setPalette(QPalette());
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::TextArea::implicitBackgroundWidth
+    \readonly
+
+    This property holds the implicit background width.
+
+    The value is equal to \c {background ? background.implicitWidth : 0}.
+
+    \sa implicitBackgroundHeight
+*/
+qreal QQuickTextArea::implicitBackgroundWidth() const
+{
+    Q_D(const QQuickTextArea);
+    if (!d->background)
+        return 0;
+    return d->background->implicitWidth();
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::TextArea::implicitBackgroundHeight
+    \readonly
+
+    This property holds the implicit background height.
+
+    The value is equal to \c {background ? background.implicitHeight : 0}.
+
+    \sa implicitBackgroundWidth
+*/
+qreal QQuickTextArea::implicitBackgroundHeight() const
+{
+    Q_D(const QQuickTextArea);
+    if (!d->background)
+        return 0;
+    return d->background->implicitHeight();
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::TextArea::topInset
+
+    This property holds the top inset for the background.
+
+    \sa {Control Layout}, bottomInset
+*/
+qreal QQuickTextArea::topInset() const
+{
+    Q_D(const QQuickTextArea);
+    return d->getTopInset();
+}
+
+void QQuickTextArea::setTopInset(qreal inset)
+{
+    Q_D(QQuickTextArea);
+    d->setTopInset(inset);
+}
+
+void QQuickTextArea::resetTopInset()
+{
+    Q_D(QQuickTextArea);
+    d->setTopInset(0, true);
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::TextArea::leftInset
+
+    This property holds the left inset for the background.
+
+    \sa {Control Layout}, rightInset
+*/
+qreal QQuickTextArea::leftInset() const
+{
+    Q_D(const QQuickTextArea);
+    return d->getLeftInset();
+}
+
+void QQuickTextArea::setLeftInset(qreal inset)
+{
+    Q_D(QQuickTextArea);
+    d->setLeftInset(inset);
+}
+
+void QQuickTextArea::resetLeftInset()
+{
+    Q_D(QQuickTextArea);
+    d->setLeftInset(0, true);
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::TextArea::rightInset
+
+    This property holds the right inset for the background.
+
+    \sa {Control Layout}, leftInset
+*/
+qreal QQuickTextArea::rightInset() const
+{
+    Q_D(const QQuickTextArea);
+    return d->getRightInset();
+}
+
+void QQuickTextArea::setRightInset(qreal inset)
+{
+    Q_D(QQuickTextArea);
+    d->setRightInset(inset);
+}
+
+void QQuickTextArea::resetRightInset()
+{
+    Q_D(QQuickTextArea);
+    d->setRightInset(0, true);
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::TextArea::bottomInset
+
+    This property holds the bottom inset for the background.
+
+    \sa {Control Layout}, topInset
+*/
+qreal QQuickTextArea::bottomInset() const
+{
+    Q_D(const QQuickTextArea);
+    return d->getBottomInset();
+}
+
+void QQuickTextArea::setBottomInset(qreal inset)
+{
+    Q_D(QQuickTextArea);
+    d->setBottomInset(inset);
+}
+
+void QQuickTextArea::resetBottomInset()
+{
+    Q_D(QQuickTextArea);
+    d->setBottomInset(0, true);
+}
+
 void QQuickTextArea::classBegin()
 {
     Q_D(QQuickTextArea);
     QQuickTextEdit::classBegin();
     d->resolveFont();
+    d->resolvePalette();
 }
 
 void QQuickTextArea::componentComplete()
@@ -636,12 +1017,13 @@ void QQuickTextArea::componentComplete()
     Q_D(QQuickTextArea);
     d->executeBackground(true);
     QQuickTextEdit::componentComplete();
+    d->resizeBackground();
 #if QT_CONFIG(quicktemplates2_hover)
     if (!d->explicitHoverEnabled)
         setAcceptHoverEvents(QQuickControlPrivate::calcHoverEnabled(d->parentItem));
 #endif
 #if QT_CONFIG(accessibility)
-    if (!d->accessibleAttached && QAccessible::isActive())
+    if (QAccessible::isActive())
         d->accessibilityActiveChanged(true);
 #endif
 }
@@ -650,20 +1032,31 @@ void QQuickTextArea::itemChange(QQuickItem::ItemChange change, const QQuickItem:
 {
     Q_D(QQuickTextArea);
     QQuickTextEdit::itemChange(change, value);
-    if ((change == ItemParentHasChanged && value.item) || (change == ItemSceneChange && value.window)) {
-        d->resolveFont();
+    switch (change) {
+    case ItemEnabledHasChanged:
+        emit paletteChanged();
+        break;
+    case ItemSceneChange:
+    case ItemParentHasChanged:
+        if ((change == ItemParentHasChanged && value.item) || (change == ItemSceneChange && value.window)) {
+            d->resolveFont();
+            d->resolvePalette();
 #if QT_CONFIG(quicktemplates2_hover)
-        if (!d->explicitHoverEnabled)
-            d->updateHoverEnabled(QQuickControlPrivate::calcHoverEnabled(d->parentItem), false); // explicit=false
+            if (!d->explicitHoverEnabled)
+                d->updateHoverEnabled(QQuickControlPrivate::calcHoverEnabled(d->parentItem), false); // explicit=false
 #endif
-        if (change == ItemParentHasChanged) {
-            QQuickFlickable *flickable = qobject_cast<QQuickFlickable *>(value.item->parentItem());
-            if (flickable) {
-                QQuickScrollView *scrollView = qobject_cast<QQuickScrollView *>(flickable->parentItem());
-                if (scrollView)
-                    d->attachFlickable(flickable);
+            if (change == ItemParentHasChanged) {
+                QQuickFlickable *flickable = qobject_cast<QQuickFlickable *>(value.item->parentItem());
+                if (flickable) {
+                    QQuickScrollView *scrollView = qobject_cast<QQuickScrollView *>(flickable->parentItem());
+                    if (scrollView)
+                        d->attachFlickable(flickable);
+                }
             }
         }
+        break;
+    default:
+        break;
     }
 }
 
@@ -674,6 +1067,14 @@ void QQuickTextArea::geometryChanged(const QRectF &newGeometry, const QRectF &ol
     d->resizeBackground();
 }
 
+void QQuickTextArea::insetChange(const QMarginsF &newInset, const QMarginsF &oldInset)
+{
+    Q_D(QQuickTextArea);
+    Q_UNUSED(newInset);
+    Q_UNUSED(oldInset);
+    d->resizeBackground();
+}
+
 QSGNode *QQuickTextArea::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *data)
 {
     Q_D(QQuickTextArea);
@@ -798,9 +1199,7 @@ void QQuickTextArea::timerEvent(QTimerEvent *event)
 class QQuickTextAreaAttachedPrivate : public QObjectPrivate
 {
 public:
-    QQuickTextAreaAttachedPrivate() : control(nullptr) { }
-
-    QQuickTextArea *control;
+    QQuickTextArea *control = nullptr;
 };
 
 QQuickTextAreaAttached::QQuickTextAreaAttached(QObject *parent)
